@@ -1,4 +1,4 @@
-/* $Id: tiff2ps.c,v 1.30 2004/10/30 13:46:33 dron Exp $ */
+/* $Id: tiff2ps.c,v 1.35 2006/02/23 14:50:32 dron Exp $ */
 
 /*
  * Copyright (c) 1988-1997 Sam Leffler
@@ -120,7 +120,7 @@ double	splitOverlap = 0;		/* amount for split pages to overlag */
 int	rotate = FALSE;			/* rotate image by 180 degrees */
 char	*filename;			/* input filename */
 int	useImagemask = FALSE;		/* Use imagemask instead of image operator */
-uint16	res_unit = 0;			/* Resolution units: 1 - inches, 2 - cm*/
+uint16	res_unit = 0;			/* Resolution units: 2 - inches, 3 - cm */
 
 /*
  * ASCII85 Encoding Support.
@@ -545,8 +545,6 @@ TIFF2PS(FILE* fd, TIFF* tif,
 	float ox, oy;
         double prw, prh;
 	double scale = 1.0;
-	double left_offset = lm * PS_UNIT_SIZE;
-	double bottom_offset = bm * PS_UNIT_SIZE;
 	uint32 subfiletype;
 	uint16* sampleinfo;
 	static int npages = 0;
@@ -597,16 +595,19 @@ TIFF2PS(FILE* fd, TIFF* tif,
 			npages++;
 			fprintf(fd, "%%%%Page: %d %d\n", npages, npages);
 			if (!generateEPSF && ( level2 || level3 )) {
-				double psw,psh;
-				if (pw!=0 && ph!=0) {
-					psw=pw;
-					psh=ph;
-			    
-				}
-				else {
+				double psw, psh;
+				if (pw != 0.0) {
+					psw = pw * PS_UNIT_SIZE;
+					if (res_unit == RESUNIT_CENTIMETER)
+						psw *= 2.54F;
+				} else
 					psw=rotate ? prh:prw;
-				    psh=rotate ? prw:prh;		    
-				}
+				if (ph != 0.0) {
+					psh = ph * PS_UNIT_SIZE;
+					if (res_unit == RESUNIT_CENTIMETER)
+						psh *= 2.54F;
+				} else
+					psh=rotate ? prw:prh;
 				fprintf(fd,
 	"1 dict begin /PageSize [ %f %f ] def currentdict end setpagedevice\n",
 					psw, psh);
@@ -616,34 +617,49 @@ TIFF2PS(FILE* fd, TIFF* tif,
 			}
 			fprintf(fd, "gsave\n");
 			fprintf(fd, "100 dict begin\n");
-			if (pw != 0 && ph != 0) {
+			if (pw != 0 || ph != 0) {
+				double psw = pw, psh = ph;
+				if (!psw)
+					psw = prw;
+				if (!psh)
+					psh = prh;
 				if (maxPageHeight) { /* used -H option */
-					split = PlaceImage(fd,pw,ph,prw,prh,0,lm,bm,cnt);
+					split = PlaceImage(fd,psw,psh,prw,prh,
+							   0,lm,bm,cnt);
 					while( split ) {
 					    PSpage(fd, tif, w, h);
 					    fprintf(fd, "end\n");
 					    fprintf(fd, "grestore\n");
 					    fprintf(fd, "showpage\n");
 					    npages++;
-					    fprintf(fd, "%%%%Page: %d %d\n", npages, npages);
+					    fprintf(fd, "%%%%Page: %d %d\n",
+						    npages, npages);
 					    fprintf(fd, "gsave\n");
 					    fprintf(fd, "100 dict begin\n");
-					    split = PlaceImage(fd,pw,ph,prw,prh,split,lm,bm,cnt);
+					    split = PlaceImage(fd,psw,psh,prw,prh,
+							       split,lm,bm,cnt);
 					}
 				} else {
+					double left_offset = lm * PS_UNIT_SIZE;
+					double bottom_offset = bm * PS_UNIT_SIZE;
+					psw *= PS_UNIT_SIZE;
+					psh *= PS_UNIT_SIZE;
+
 					/* NB: maintain image aspect ratio */
-					scale = (pw*PS_UNIT_SIZE/prw) < (ph*PS_UNIT_SIZE/prh) ?
-					    (pw*PS_UNIT_SIZE/prw) :
-					    (ph*PS_UNIT_SIZE/prh);
+					scale = psw/prw < psh/prh ?
+						psw/prw : psh/prh;
 					if (scale > 1.0)
 						scale = 1.0;
-					bottom_offset +=
-						(ph * PS_UNIT_SIZE - prh * scale) / (cnt?2:1);
-					if (cnt)
-						left_offset += (pw * PS_UNIT_SIZE - prw * scale) / 2;
+					if (cnt) {
+						bottom_offset +=
+							(psh - prh * scale) / 2;
+						left_offset +=
+							(psw - prw * scale) / 2;
+					}
 					fprintf(fd, "%f %f translate\n",
 						left_offset, bottom_offset);
-					fprintf(fd, "%f %f scale\n", prw * scale, prh * scale);
+					fprintf(fd, "%f %f scale\n",
+						prw * scale, prh * scale);
 					if (rotate)
 						fputs ("1 1 translate 180 rotate\n", fd);
 				}
@@ -2080,3 +2096,5 @@ usage(int code)
 		fprintf(stderr, "%s\n", stuff[i]);
 	exit(code);
 }
+
+/* vim: set ts=8 sts=8 sw=8 noet: */
